@@ -1,4 +1,4 @@
-/*	$OpenBSD: config.c,v 1.102 2022/06/05 12:43:13 claudio Exp $ */
+/*	$OpenBSD: config.c,v 1.104 2022/08/17 15:15:25 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004, 2005 Henning Brauer <henning@openbsd.org>
@@ -85,6 +85,14 @@ copy_config(struct bgpd_config *to, struct bgpd_config *from)
 	to->fib_priority = from->fib_priority;
 }
 
+void
+network_free(struct network *n)
+{
+	rtlabel_unref(n->net.rtlabel);
+	filterset_free(&n->net.attrset);
+	free(n);
+}
+
 void
 free_networks(struct network_head *networks)
 {
@@ -92,8 +100,7 @@ free_networks(struct network_head *networks)
 
 	while ((n = TAILQ_FIRST(networks)) != NULL) {
 		TAILQ_REMOVE(networks, n, entry);
-		filterset_free(&n->net.attrset);
-		free(n);
+		network_free(n);
 	}
 }
 
@@ -402,7 +409,7 @@ host(const char *s, struct bgpd_addr *h, uint8_t *len)
 		p[0] = '\0';
 	}
 
-	bzero(h, sizeof(*h));
+	memset(h, 0, sizeof(*h));
 
 	if (host_ip(ps, h, len) == 0) {
 		free(ps);
@@ -422,7 +429,7 @@ host_ip(const char *s, struct bgpd_addr *h, uint8_t *len)
 	struct addrinfo		 hints, *res;
 	int			 bits;
 
-	bzero(&hints, sizeof(hints));
+	memset(&hints, 0, sizeof(hints));
 	hints.ai_family = AF_UNSPEC;
 	hints.ai_socktype = SOCK_DGRAM; /*dummy*/
 	hints.ai_flags = AI_NUMERICHOST;
@@ -431,7 +438,8 @@ host_ip(const char *s, struct bgpd_addr *h, uint8_t *len)
 		sa2addr(res->ai_addr, h, NULL);
 		freeaddrinfo(res);
 	} else {	/* ie. for 10/8 parsing */
-		if ((bits = inet_net_pton(AF_INET, s, &h->v4, sizeof(h->v4))) == -1)
+		if ((bits = inet_net_pton(AF_INET, s, &h->v4,
+		    sizeof(h->v4))) == -1)
 			return (0);
 		*len = bits;
 		h->aid = AID_INET;
@@ -511,10 +519,9 @@ prepare_listeners(struct bgpd_config *conf)
 }
 
 void
-expand_networks(struct bgpd_config *c)
+expand_networks(struct bgpd_config *c, struct network_head *nw)
 {
 	struct network		*n, *m, *tmp;
-	struct network_head	*nw = &c->networks;
 	struct prefixset	*ps;
 	struct prefixset_item	*psi;
 
@@ -536,8 +543,7 @@ expand_networks(struct bgpd_config *c)
 				    &m->net.attrset);
 				TAILQ_INSERT_TAIL(nw, m, entry);
 			}
-			filterset_free(&n->net.attrset);
-			free(n);
+			network_free(n);
 		}
 	}
 }
