@@ -42,7 +42,11 @@
 #include "bgpd.h"
 #include "log.h"
 
-#define	RTP_KERN	0x01
+#define	RTP_ANY		0x0
+#define	RTP_KERN	0x1
+#define	RTP_PROTO1	0x11
+#define	RTP_PROTO2	0x12
+#define	RTP_PROTO3	0x13
 #define	RTP_MINE	0xff
 
 struct ktable		**krt;
@@ -250,6 +254,20 @@ kr_init(int *fd, uint8_t fib_prio)
 	return (0);
 }
 
+int
+kr_default_prio(void)
+{
+	return RTP_PROTO3;
+}
+
+int
+kr_check_prio(long long prio)
+{
+	if (prio < RTP_PROTO1 || prio > RTP_PROTO3)
+		return 0;
+	return 1;
+}
+
 int
 ktable_new(u_int rtableid, u_int rdomid, char *name, int fs)
 {
@@ -2503,19 +2521,26 @@ get_mpe_config(const char *name, u_int *rdomain, u_int *label)
 static int
 prio2flags(uint8_t fib_prio)
 {
-	/* XXX this needs to be configurable */
-	return RTF_PROTO3;
+	switch (fib_prio) {
+	case RTP_PROTO1:
+		return RTF_PROTO1;
+	case RTP_PROTO2:
+		return RTF_PROTO2;
+	default:
+	case RTP_PROTO3:
+		return RTF_PROTO3;
+	}
 }
 
 static uint8_t
 flags2prio(int flags)
 {
 	uint8_t	prio = RTP_KERN;
+	int prioflag;
 
 	/* Everything from the kernel is one prio unless it is a bgpd route */
-	/* XXX this needs to be configurable */
-
-	if (flags & RTF_PROTO3)
+	prioflag = prio2flags(kr_state.fib_prio);
+	if ((flags & prioflag) == prioflag)
 		prio = RTP_MINE;
 
 	return prio;
