@@ -1,4 +1,4 @@
-/*	$OpenBSD: rde_update.c,v 1.147 2022/09/01 13:19:11 claudio Exp $ */
+/*	$OpenBSD: rde_update.c,v 1.148 2022/09/23 15:49:20 claudio Exp $ */
 
 /*
  * Copyright (c) 2004 Claudio Jeker <claudio@openbsd.org>
@@ -361,6 +361,113 @@ up_generate_addpath(struct filter_head *rules, struct rde_peer *peer,
 	}
 }
 
+/*
+ * Generate updates for the add-path send all case. Since all prefixes
+ * are distributed just remove old and add new.
+ */ 
+void
+up_generate_addpath_all(struct filter_head *rules, struct rde_peer *peer,
+    struct prefix *best, struct prefix *new, struct prefix *old)
+{
+	struct filterstate	state;
+	struct bgpd_addr	addr;
+	struct prefix		*p, *next, *head = NULL;
+	uint8_t			prefixlen;
+	int			all = 0;
+
+	/*
+	 * if old and new are NULL then insert all prefixes from best,
+	 * clearing old routes in the process
+	 */
+	if (old == NULL && new == NULL) {
+		/* mark all paths as stale */
+		pt_getaddr(best->pt, &addr);
+		prefixlen = best->pt->prefixlen;
+
+		head = prefix_adjout_lookup(peer, &addr, prefixlen);
+		for (p = head; p != NULL; p = prefix_adjout_next(peer, p))
+			p->flags |= PREFIX_FLAG_STALE;
+
+		new = best;
+		all = 1;
+	}
+
+	if (old != NULL) {
+		/* withdraw stale paths */
+		pt_getaddr(old->pt, &addr);
+		p = prefix_adjout_get(peer, old->path_id_tx, &addr,
+		    old->pt->prefixlen);
+		if (p != NULL)
+			prefix_adjout_withdraw(p);
+	}
+
+	if (new != NULL) {
+		pt_getaddr(new->pt, &addr);
+		prefixlen = new->pt->prefixlen;
+	}
+
+	/* add new path (or multiple if all is set) */
+	for (; new != NULL; new = next) {
+		if (all)
+			next = TAILQ_NEXT(new, entry.list.rib);
+		else
+			next = NULL;
+
+		/* only allow valid prefixes */
+		if (!prefix_eligible(new))
+			break;
+
+		/*
+		 * up_test_update() needs to run before the output filters
+		 * else the well known communities won't work properly.
+		 * The output filters would not be able to add well known
+		 * communities.
+		 */
+		if (!up_test_update(peer, new))
+			continue;
+
+		rde_filterstate_prep(&state, prefix_aspath(new),
+		    prefix_communities(new), prefix_nexthop(new),
+		    prefix_nhflags(new));
+		if (rde_filter(rules, peer, prefix_peer(new), &addr,
+		    prefixlen, prefix_vstate(new), &state) == ACTION_DENY) {
+			rde_filterstate_clean(&state);
+			continue;
+		}
+
+		if (up_enforce_open_policy(peer, &state)) {
+			rde_filterstate_clean(&state);
+			continue;
+		}
+
+		/* from here on we know this is an update */
+		p = prefix_adjout_get(peer, new->path_id_tx, &addr, prefixlen);
+
+		up_prep_adjout(peer, &state, addr.aid);
+		prefix_adjout_update(p, peer, &state, &addr,
+		    prefixlen, new->path_id_tx, prefix_vstate(new));
+		rde_filterstate_clean(&state);
+
+		/* max prefix checker outbound */
+		if (peer->conf.max_out_prefix &&
+		    peer->prefix_out_cnt > peer->conf.max_out_prefix) {
+			log_peer_warnx(&peer->conf,
+			    "outbound prefix limit reached (>%u/%u)",
+			    peer->prefix_out_cnt, peer->conf.max_out_prefix);
+			rde_update_err(peer, ERR_CEASE,
+			    ERR_CEASE_MAX_SENT_PREFIX, NULL, 0);
+		}
+	}
+
+	if (all) {
+		/* withdraw stale paths */
+		for (p = head; p != NULL; p = prefix_adjout_next(peer, p)) {
+			if (p->flags & PREFIX_FLAG_STALE)
+				prefix_adjout_withdraw(p);
+		}
+	}
+}
+
 struct rib_entry *rib_add(struct rib *, struct bgpd_addr *, int);
 void rib_remove(struct rib_entry *);
 int rib_empty(struct rib_entry *);
